""" State Reporting Library specifically for the TX report zone.
"""
import logging

from cpsar.srlib import add2report
from cpsar.util import spawn_sftp

log = logging.getLogger('')

def info(msg, *args):
    """ Logging helper. Send the information to the output log and save on the
    report 
    """
    log.info(msg, *args)
    add2report('TX', msg, *args)

@property
def remote_host(self):
    if self.opts.mode == 'P':
        return 'sftpp.tdi.state.tx.us'
    else:
        return 'sftpt.tdi.state.tx.us'

def sftp_connect(self):
    return spawn_sftp(self.remote_host, 'edi631040950', 'NrVZYTa4')

def setup_options(self):
    super(type(self), self).setup_options()
    self.add_option('-m', '--mode', help='mode of operation: T or P',
                    default='P')

x824_ack_codes = {
    'A': 'Accept',
    'E': 'Accept with Errors',
    'R': 'Reject'
}

x824_error_codes = {
    '001' :	'Mandatory field not present',
    '028' :	'Must be numeric (0-9)',
    '029' :	'Must be a valid date (CCYYMMDD)',
    '030' :	'Must be A-Z, 0-9, or spaces',
    '033' :	'Must be <= Date of Injury',
    '034' :	'Must be >= Date of Injury',
    '039' :	'No match on database',
    '040' :	'All digits cannot be the same',
    '041' :	'Must be <= Current date',
    '042' :	'Not statutorily valid',
    '044' :	'Value is > than required by jurisdiction',
    '045' :	'Value is < than required by jurisdiction',
    '050' :	'No matching Subsequent report (A49)',
    '053' :	'No matching FROI (148)',
    '054' :	'Must be valid occurrence for segment',
    '057' :	'Duplicate transmission/transaction',
    '058' :	'Code/ID invalid',
    '059' :	'Value not consistent with value previously reported',
    '061' :	'Event Criteria not met',
    '062' :	'Required segment not present',
    '063' :	'Invalid event sequence/relationship',
    '064' :	'Invalid data sequence/relationship',
    '065' :	'Corresponding report/data not found',
    '066' :	'Invalid record count',
    '070' :	"Must be <= 'From' Service Date",
    '071' :	"Must be >= 'Thru' Service Date",
    '072' :	"Must be > Date of Bill",
    '073' :	"Must be >= Date payer received bill",
    '074' :	"Must be >= 'From' Service Date",
    '075' :	"Must be <= 'Thru' Service Date"
}

x824_req_fields = {
    "DN 05" :	"Jurisdiction Claim Number",
    "DN 06" :	"Insurer FEIN",
    "DN 07" :	"Insurer Name",
    "DN 14" :	"Claim Administrator Postal Code",
    "DN 15" :	"Claim Administrator Claim Number",
    "DN 16" :	"Employer FEIN",
    "DN 18" :	"Employer Name",
    "DN 19" :	"Employer Physical Primary Address",
    "DN 20" :	"Employer Physical Secondary Address",
    "DN 21" :	"Employer Physical City",
    "DN 22" :	"Employer Physical State Code",
    "DN 23" :	"Employer Physical Postal Code",
    "DN 28" :	"Policy Number",
    "DN 31" :	"Date of Injury",
    "DN 42" :	"Employee Social Security Number",
    "DN 43" :	"Employee Last Name",
    "DN 44" :	"Employee First Name",
    "DN 45" :	"Employee Middle Name/Initial",
    "DN 46" :	"Employee Mailing Primary Address",
    "DN 47" :	"Employee Mailing Secondary Address",
    "DN 48" :	"Employee Mailing City",
    "DN 49" :	"Employee Mailing State Code",
    "DN 50" :	"Employee Mailing Postal Code",
    "DN 51" :	"Employee Phone",
    "DN 52" :	"Employee Date of Birth",
    "DN 53" :	"Employee Gender Code",
    "DN 54" :	"Employee Marital Status Code",
    "DN 98" :	"Sender ID",
    "DN 99" :	"Receiver ID",
    "DN 100" :	"Date Transmission Sent",
    "DN 101" :	"Time Transmission Sent",
    "DN 152" :	"Employee Employment Visa",
    "DN 153" :	"Employee Green Card",
    "DN 154" :	"Employee ID Assigned by Jurisdiction",
    "DN 155" :	"Employee Mailing Country Code",
    "DN 156" :	"Employee Passport Number",
    "DN 159" :	"Employer Contact Business Phone Number",
    "DN 164" :	"Employer Physical Country Code",
    "DN 187" :	"Claim Administrator FEIN",
    "DN 188" :	"Claim Administrator Name",
    "DN 208" :	"Managed Care Organization ID",
    "DN 209" :	"Managed Care Organization Name",
    "DN 255" :	"Employee Last Name Suffix",
    "DN 266" :	"Transaction Tracking Number",
    "DN 500" :	"Unique Bill ID Number",
    "DN 501" :	"Total Charge Per Bill",
    "DN 502" :	"Billing Type Code",
    "DN 503" :	"Billing Format Code",
    "DN 504" :	"Facility Code",
    "DN 505" :	"Bill Frequency Type Code",
    "DN 506" :	"Provider Signature on File Indicator",
    "DN 507" :	"Provider Agreement Code",
    "DN 508" :	"Bill Submission Reason Code",
    "DN 509" :	"Service Bill Date(s) Range",
    "DN 510" :	"Date of Bill",
    "DN 511" :	"Date Insurer Received Bill",
    "DN 512" :	"Date Insurer Paid Bill",
    "DN 513" :	"Admission Date",
    "DN 514" :	"Discharge Date",
    "DN 515 " :	"Contract Type Code",
    "DN 516" :	"Total Amount Paid Per Bill",
    "DN 517" :	"Patient Account Number",
    "DN 518" :	"DRG Code",
    "DN 521" :	"Principal Diagnosis Code",
    "DN 522" :	"ICD-9 Diagnosis Code",
    "DN 523" :	"Billing Provider Unique Bill Identification",
    "DN 524" :	"Procedure Date",
    "DN 525" :	"ICD-9 Principal Procedure Code",
    "DN 526" :	"Release of Information Code",
    "DN 527" :	"Prescription Bill Date",
    "DN 528" :	"Billing Provider Last/Group Name",
    "DN 529" :	"Billing Provider First Name",
    "DN 530" :	"Billing Provider Middle Name/Initial",
    "DN 531" :	"Billing Provider Last Name Suffix",
    "DN 532" :	"Batch Control Number",
    "DN 534" :	"Gatekeeper Indicator",
    "DN 535" :	"Admitting Diagnosis Code",
    "DN 537" :	"Billing Provider Primary Specialty Code",
    "DN 538" :	"Billing Provider Primary Address",
    "DN 539" :	"Billing Provider Secondary Address",
    "DN 540" :	"Billing Provider City",
    "DN 541" :	"Billing Provider State Code",
    "DN 542" :	"Billing Provider Postal Code",
    "DN 543" :	"Bill Adjustment Group Code",
    "DN 544" :	"Bill Adjustment Reason Code",
    "DN 545" :	"Bill Adjustment Amount",
    "DN 546" :	"Bill Adjustment Units",
    "DN 547" :	"Line Number",
    "DN 550" :	"Principal Procedure Date",
    "DN 551" :	"Procedure Description",
    "DN 552" :	"Total Charge Per Line",
    "DN 553" :	"Day(s) Unit(s) Code",
    "DN 554" :	"Day(s) Unit(s) Billed",
    "DN 555" :	"Place of Service Bill Code",
    "DN 557" :	"Diagnosis Pointer",
    "DN 559" :	"Revenue Billed Code",
    "DN 560" :	"Revenue Unit Rate",
    "DN 561" :	"Prescription Line Number",
    "DN 562" :	"Dispense as Written Code",
    "DN 563" :	"Drug Name",
    "DN 564" :	"Basis of Cost Determination Code",
    "DN 565" :	"Total Charge Per Line - Rental",
    "DN 566" :	"Total Charge Per Line - Purchase",
    "DN 567" :	"DME Billing Frequency Code",
    "DN 568" :	"CRNA Supervision Indicator",
    "DN 569" :	"Billing Provider Country Code",
    "DN 570" :	"Drugs/Supplies Quantity Dispensed",
    "DN 571" :	"Drugs/Supplies Number of Days",
    "DN 572" :	"Drugs/Supplies Billed Amount",
    "DN 574" :	"Total Amount Paid Per Line",
    "DN 576" :	"Revenue Paid Code",
    "DN 577" :	"Admission Type Code",
    "DN 579" :	"Drugs/Supplies Dispensing Fee",
    "DN 580" :	"Day(s)/Unit(s) Paid",
    "DN 581" :	"Treatment Authorization Number ",
    "DN 583" :	"Rendering Line Provider Anesthesia Lic Number",
    "DN 584" :	"Rendering Line Provider City",
    "DN 585" :	"Rendering Line Provider Country Code",
    "DN 586" :	"Rendering Line Provider FEIN",
    "DN 587" :	"Rendering Line Provider First Name",
    "DN 588" :	"Rendering Line Provider Last Name Suffix",
    "DN 589" :	"Rendering Line Provider Last/Group Name",
    "DN 590" :	"Rendering Line Provider Medicare Number",
    "DN 591" :	"Rendering Line Provider Middle Name/Initial",
    "DN 592" :	"Rendering Line Provider National Provider ID",
    "DN 593" :	"Rendering Line Provider Postal Code",
    "DN 594" :	"Rendering Line Provider Primary Address",
    "DN 595" :	"Rendering Line Provider Primary Specialty Code",
    "DN 596" :	"Rendering Line Provider Secondary Address",
    "DN 597" :	"Rendering Line Provider Specialty License Number",
    "DN 598" :	"Rendering Line Provider State Code",
    "DN 599" :	"Rendering Line Provider State License Number",
    "DN 600" :	"Place of Service Line Code",
    "DN 604" :	"Prescription Line Date",
    "DN 605" :	"Service Line Date(s) Range",
    "DN 615" :	"Reporting Period",
    "DN 616" :	"Insurer Postal Code",
    "DN 622" :	"Admission Hour",
    "DN 623" :	"Discharge Hour",
    "DN 624" :	"Initial Amount Paid",
    "DN 626" :	"HCPCS Principle Procedure Billed Code",
    "DN 629" :	"Billing Provider FEIN",
    "DN 630" :	"Billing Provider State License Number",
    "DN 632" :	"Billing Provider Medicare Number",
    "DN 633" :	"Billing Provider Anesthesia License Number",
    "DN 634" :	"Billing Provider National Provider ID",
    "DN 636" :	"Billing Provider Specialty License Number",
    "DN 638" :	"Rendering Bill Provider Last/Group Name",
    "DN 639" :	"Rendering Bill Provider First Name",
    "DN 640" :	"Rendering Bill Provider Middle Name/Initial",
    "DN 641" :	"Rendering Bill Provider Last Name Suffix",
    "DN 642" :	"Rendering Bill Provider FEIN",
    "DN 643" :	"Rendering Bill Provider State License Number",
    "DN 645" :	"Rendering Bill Provider Medicare Number",
    "DN 646" :	"Rendering Bill Provider Anesthesia License Number",
    "DN 647" :	"Rendering Bill Provider National Provider ID",
    "DN 649" :	"Rendering Bill Provider Specialty License Number",
    "DN 651" :	"Rendering Bill Provider Primary Specialty Code",
    "DN 652 " :	"Rendering Bill Provider Primary Address",
    "DN 653" :	"Rendering Bill Provider Secondary Address",
    "DN 654" :	"Rendering Bill Provider City",
    "DN 655" :	"Rendering Bill Provider State Code",
    "DN 656" :	"Rendering Bill Provider Postal Code",
    "DN 657" :	"Rendering Bill Provider Country Code",
    "DN 658" :	"Supervising Provider Last/Group Name",
    "DN 659" :	"Supervising Provider First Name",
    "DN 660" :	"Supervising Provider Middle Name/Initial",
    "DN 661" :	"Supervising Provider Last Name Suffix",
    "DN 662" :	"Supervising Provider FEIN",
    "DN 663" :	"Supervising Provider State License Number",
    "DN 665" :	"Supervising Provider Medicare Number",
    "DN 666" :	"Supervising Provider Anesthesia License Number",
    "DN 667" :	"Supervising Provider National Provider ID",
    "DN 669" :	"Supervising Provider Specialty License Number",
    "DN 671" :	"Supervising Provider Primary Specialty Code",
    "DN 672" :	"Supervising Provider Primary Address",
    "DN 673 " :	"Supervising Provider Secondary Address",
    "DN 674" :	"Supervising Provider City",
    "DN 675" :	"Supervising Provider State Code",
    "DN 676" :	"Supervising Provider Postal Code",
    "DN 677" :	"Supervising Provider Country Code",
    "DN 678" :	"Facility Name",
    "DN 679" :	"Facility FEIN",
    "DN 680" :	"Facility State License Number",
    "DN 681" :	"Facility Medicare Number",
    "DN 682" :	"Facility National Provider ID",
    "DN 684" :	"Facility Primary Address",
    "DN 685" :	"Facility Secondary Address",
    "DN 686" :	"Facility City",
    "DN 687" :	"Facility State Code",
    "DN 688" :	"Facility Postal Code",
    "DN 689" :	"Facility Country Code",
    "DN 690" :	"Referring Provider Last/Group Name",
    "DN 691" :	"Referring Provider First Name",
    "DN 692" :	"Referring Provider Middle Name/Initial",
    "DN 693" :	"Referring Provider Last Name Suffix",
    "DN 694" :	"Referring Provider FEIN",
    "DN 695" :	"Referring Provider State License Number",
    "DN 697" :	"Referring Provider Medicare Number",
    "DN 698" :	"Referring Provider Anesthesia License Number",
    "DN 699" :	"Referring Provider National Provider ID",
    "DN 701" :	"Referring Provider Specialty License Number",
    "DN 704" :	"Managed Care Organization FEIN",
    "DN 708" :	"Managed Care Organization Primary Address",
    "DN 709" :	"Managed Care Organization Secondary Address",
    "DN 710" :	"Managed Care Organization City",
    "DN 711" :	"Managed Care Organization State Code",
    "DN 712" :	"Managed Care Organizaton Postal Code",
    "DN 713" :	"Managed Care Organization Country Code",
    "DN 714" :	"HCPCS Line Procedure Billed Code",
    "DN 715" :	"Jurisdiction Procedure Billed Code",
    "DN 717" :	"HCPCS Modifier Billed Code",
    "DN 718" :	"Jurisdiction Modifier Billed Code",
    "DN 719 " :	"ADA Procedure Billed Code",
    "DN 721" :	"NDC Billed Code",
    "DN 722" :	"ADA Procedure Paid Code",
    "DN 726" :	"HCPCS Line Procedure Paid Code",
    "DN 727" :	"HCPCS Modifier Paid Code",
    "DN 728" :	"NDC Paid Code",
    "DN 729" :	"Jurisdiction Procedure Paid Code",
    "DN 730" :	"Jurisdiction Modifier Paid Code",
    "DN 731" :	"Service Adjustment Group Code",
    "DN 732" :	"Service Adjustment Reason Code",
    "DN 733" :	"Service Adjustment Amount",
    "DN 734" :	"Service Adjustment Units",
    "DN 736" :	"ICD-9 CM Procedure Code",
    "DN 737" :	"HCPCS Bill Procedure Code",
    "DN 738" :	"Treatment Line Authorization Number",
    "DN 741" :	"Contract Line Type Code",
    "DN 742" :	"Provider Agreement Line Code"
}


ak5_status_map = {
    'A': 'Accepted',
    'E': 'Accepted But Errors Were Noted',
    'R': 'Rejected'
}
ak5_message_map = {
    '1': 'Transaction Set Not Supported',
    '2': 'Transaction Set Trailer Missing',
    '3': 'Transaction Set Control Number in ' +
         'Header and Trailer Do Not Match',
    '4': 'Number of Included Segments Does Not Match Actual Count',
    '5': 'One or More Segments in Error',
    '6': '6 Missing or Invalid Transaction Set Identifier',
    '7': 'Missing or Invalid Transaction Set Control Number',
   '23': 'Transaction Set Control Number Not Unique within ' +
         'the Functional Group'
}
ak9_status_map = {
    'A': 'Accepted',
    'E': 'Accepted, But Errors Were Noted.',
    'P': 'Partially Accepted, At Least One Transaction Set Was Rejected',
    'R': 'Rejected'
}
ak9_syntax_error_map = {
    '1': 'Functional Group Not Supported',
    '2': 'Functional Group Version Not Supported',
    '3': 'Functional Group Trailer Missing',
    '4': 'Group Control Number in the Functional Group Header and '
         'Trailer Do Not Agree',
    '5': 'Number of Included Transaction Sets Does Not Match Actual Count',
    '6': 'Group Control Number Violates Syntax'
}
